// -*-C-*-
/*!
 * @file rtc_main.c
 * @brief RTUnit component base functions
 * @author Noriaki Ando <n-ando@aist.go.jp>
 *
 * Copyright (C) 2005 Noriaki Ando
 *     Task-intelligence Research Graoup,
 *     Intelligent Systems Research Institute,
 *     National Institute of
 *         Advanced Industrial Science and Technology (AIST), Japan
 *     All rights reserved.
 *
 * $Id$
 */

#include "rtc_base.h"
#include "rtc_msg.h"

//============================================================
// RTC main loop
//
// Invoke state process functions
//============================================================
void rtc_mainloop()
{
  RtmRes res;
  ComponentState cur_state, nxt_state;
  cur_state = g_CurrState;
  nxt_state = g_NextState;
  g_CurrState = nxt_state;

  if (cur_state != nxt_state)
    {
      res = _rtc_state_exit(cur_state);
      _check_error(res);

      // Update component state for StatePort
      //      m_StatePort.write();
      nxt_state = g_NextState; // update next state

      // Enter to the next state.
      // If next state is transient state, execute transient state function.
      res = _rtc_state_entry(nxt_state);
//		putc(0xFF);
//		putc(0xFF);
      _check_error(res);
    }
  else
    {
      // ֤ϤΤޤޤʤΤdo ¹
      res = _rtc_state_do(cur_state);
      _check_error(res);
    }
  return;
}


//============================================================
// Check state process function return value
//============================================================
RtmRes _check_error(RtmRes result)
{
  /*if (result == RTM_ERR && g_CurrState == RTC_RESETTING)
	{
	putc(0x20);
	g_NextState = RTC_ERROR;
	return RTM_OK;
	}

if (result == RTM_OK && g_CurrState == RTC_RESETTING)
	{
	putc(0x21);
	g_NextState = RTC_INACTIVE;
	return RTM_OK;
	}
*/
  // If current function returns error, goto RTC_ERROR status.
  if (result == RTM_ERR && g_CurrState == RTC_ACTIVE)
    {
      g_NextState = RTC_ERROR;
      return RTM_ERR;
    }
  //
  // Active state have to transit RTC_ABORTING status.
  //
 /* else if (result == RTM_ERR
           && g_CurrState == RTC_ACTIVE)
    {
      g_NextState = RTC_ABORTING;
      return RTM_ERR;
    }
  *///
  // If function returns RTM_FATAL_ERR, status is directly changed
  //  RTC_FATAL_ERRROR
  else if (result == RTM_FATAL_ERR)
    {
     // g_NextState = RTC_FATAL_ERROR;
		g_NextState = RTC_ERROR;
      return RTM_FATAL_ERR;
    }

  return RTM_OK;
}


//============================================================
// Connect proxy
//============================================================
void rtc_connect_proxy()
{
  while(!bkbhit)
    {
      //_rtc_profile();
	puts("CONNECT");
      //puts(0x53);
      //putc(0x41);
      delay_ms(1000);
    }

  while(bkbhit)
    {
      bgetc();
      delay_us(10);
    }

  _rtc_init();
}


//============================================================
// Parse RTC command: Level1
//============================================================
void rtc_parse_cmd()
{
  switch (rtc_msg_get(CMD_LEVEL1))
    {
    case RTC_START:
	//putc(0x00);
	RTM_ACK_OK;
	_rtc_start();
	break;
    case RTC_STOP:
	RTM_ACK_OK;
	_rtc_stop();
	break;
    case RTC_RESET:
	RTM_ACK_OK;
	_rtc_reset();
	break;
    case RTC_ERR_TEST:
	RTM_ACK_OK;
	_rtc_error();
	break;
	//-------------------
    case RTC_INACTIVE:
	//putc(0x05);
	RTM_ACK_OK;
	g_NextState = RTC_INACTIVE;
	break;
    case RTC_ACTIVE:
	//putc(0x04);
	RTM_ACK_OK;
	g_NextState = RTC_ACTIVE;
	break; 
//    case RTC_EXIT:
//      _rtc_exit(); break;
//    case RTC_KILL:
//      _rtc_kill(); break;
    case RTC_STATE:
	_rtc_state(); break;
    case RTC_INPORTS:
	_rtc_inports(); break;
    case RTC_OUTPORTS:
	_rtc_outports(); break;
    default:
	break;
    }
}


//============================================================
// Send RT-Component profile
//============================================================
void _rtc_profile()
{
  int i;
  //rtc_msg_begin();
  //rtc_msg_putc(RTC_PROFILE);
  for (i = 0; i < RTC_PROFILE_LEN; i++)
    {
      //rtc_msg_putc(read_eeprom(i));
      putc(read_eeprom(i));
    }
  delay_ms(500);
  //rtc_msg_putc(0x41);
  //rtc_msg_end();
}


//============================================================
// Initialize RT-Component's state
//============================================================
void _rtc_init()
{
  g_CurrState = RTC_CREATED;
  g_NextState = RTC_STOPPED;
}


//============================================================
// Return current state to proxy component
//============================================================
void _rtc_state()
{
}


//============================================================
// Return InPort list to proxy component
//============================================================
void _rtc_inports()
{
}


//============================================================
// Return OutPort list to proxy component
//============================================================
void _rtc_outports()
{
}


//============================================================
// Shutdown this component
//============================================================
void _rtc_shutdown()
{
}

//=========================================================
// Send ACK
//=========================================================
void rtm_ack(BYTE ret)
{
  rtc_msg_begin();			// send HEADER
  rtc_msg_putc(RTC_ACK);	// send command
  rtc_msg_putc(ret);		// send OK or ERR
  rtc_msg_end();			// send check sum
}


//============================================================
// Activate component
//============================================================
RtmRes _rtc_start()
{
  if (g_CurrState == RTC_STOPPED)
    { 
      //rtc_outport_init();

	  //outp_d =   BYTE *g_monitorOut
      g_NextState = RTC_RUNNING;
	//RTM_OK_RETURN;
	//RTC_START = 0x00
	return RTM_OK;
    }
   else	RTM_ACK_ERR;
 return RTM_ERR;
}


//============================================================
// Deactivate component
//============================================================
RtmRes _rtc_stop()
{
  if (g_CurrState == RTC_ACTIVE)
    {
      g_NextState = RTC_STOPPED;
      RTM_ACK_OK;
	  return RTM_OK;
    }
  else RTM_ACK_ERR;
  return RTM_ERR;
}


//============================================================
// Reset component
//============================================================
RtmRes _rtc_reset()
{
	if(g_CurrState == RTC_ERROR)
	{
		g_NextState = RTC_RESETTING;
		RTM_ACK_OK;
		return RTM_OK;
	}  
  else RTM_ACK_ERR;
  return RTM_ERR;
}

RtmRes _rtc_error()
{
	g_NextState = RTC_ERROR;
	RTM_ACK_OK;
	return RTM_OK;
}
//============================================================
// Exit component
//============================================================
/*RtmRes _rtc_exit()
{

  if (g_CurrState == RTC_INACTIVE || g_CurrState == RTC_ERROR)
    {
      g_NextState = RTC_EXITING;
      RTM_ACK_OK;
    }
  RTM_ACK_ERR;
}
*/

//============================================================
// Kill component
//============================================================
/*RtmRes _rtc_kill()
{

  if (g_CurrState == RTC_FATAL_ERROR)
    {
      g_NextState = RTC_EXITING;
    }
  else
    {
      _rtc_shutdown();
      return RTM_OK;
    }
  return RTM_ERR;
}
*/

//============================================================
// Do-state functions invocation
//============================================================
RtmRes _rtc_state_do(ComponentState state)
{
  switch (state)
    {
	case RTC_RUNNING:
		g_NextState = RTC_INACTIVE;
		break;
    case RTC_ACTIVE:
      	return rtc_on_execute();
        break;
	case RTC_ERROR:
		return rtc_on_error();
		break;
	case RTC_INACTIVE:
		return rtc_on_deactivate();
		break;

	default:
		return RTM_OK;
		break;
    }
  return RTM_OK;
}


//============================================================
// Entry-state functions invocation
//============================================================
RtmRes _rtc_state_entry(ComponentState state)
{
  switch(state)
    {
	case RTC_RUNNING:
	 g_NextState = RTC_INACTIVE;
	 break;
	case RTC_ACTIVE:
	  //RTC_ACTIVE = 0x05
	  return rtc_on_activate();
	  break;
	case RTC_INACTIVE:
	  //RTC_INACTIVE = 0x04
	  break;
	case RTC_RESETTING:
	 // putc(0x22);
	  if (rtc_on_reset() == RTM_OK)
		{
			g_NextState = RTC_INACTIVE;
			return RTM_OK;
		}
	  else
		{
			g_NextState = RTC_ERROR;
			return RTM_OK;
		}
	  break;
	  
  default:
      return RTM_OK;
      break;
    }
  return RTM_OK;
}


//============================================================
// Exit-state functions invocation
//============================================================
RtmRes _rtc_state_exit(ComponentState state)
{
  switch(state)
    {
	case RTC_CREATED: //t.suzuki
	  return rtc_on_initialize();
	  break;
	case RTC_STOPPED: //t.suzuki
	  return rtc_on_startup();
	  break;	
	case RTC_RUNNING:
	  if(g_NextState == RTC_STOPPED)
		{
			return rtc_on_shutdown();
			break;
		}
	  break;
	case RTC_ACTIVE:
	  if(g_NextState == RTC_ERROR)
	  	{
			return rtc_on_aborting();
			break;
		}
	  else if(g_NextState == RTC_INACTIVE)
		{
			return rtc_on_deactivate();
			break;
		}
	  return RTM_ERR;
	  break;

    default:
      return RTM_OK;
      break;
    }
  return RTM_OK;
}


